package org.pimpmyshirt.domain;

/**
 * Container for rating information associated with a shirt.
 */
public class ShirtRating extends Entity {

	private int numberOfLowVotes;
	private int numberOfMediumVotes;
	private int numberOfHighVotes;
	private Shirt shirt;
	
	public ShirtRating(Shirt shirt) {
		this.shirt=shirt;
	}

	/**
	 * Cast a vote for a particular rating of the shirt.
	 */
	public void vote(Rating rating) {
		switch (rating) {
		case LOW:
			numberOfLowVotes++;
			break;
		case MEDIUM:
			numberOfMediumVotes++;
			break;
		default:
			numberOfHighVotes++;
		}
	}

	/**
	 * Returns the number of votes cast for a specified
	 * rating of this shirt.
	 */
	public int getNumberOfVotes(Rating rating) {
		switch (rating) {
		case LOW:
			return numberOfLowVotes;
		case MEDIUM:
			return numberOfMediumVotes;
		default:
			return numberOfHighVotes;
		}
	}
	
	public int getTotalNumberOfVotes() {
		return numberOfLowVotes + numberOfMediumVotes + numberOfHighVotes;
	}
	
	public int getNumberOfLowVotes() {
		return numberOfLowVotes;
	}
	
	public int getNumberOfMediumVotes() {
		return numberOfMediumVotes;
	}
	
	public int getNumberOfHighVotes() {
		return numberOfHighVotes;
	}

	public Shirt getShirt() {
		return shirt;
	}
}
